$TITLE  Model of Imported Imported Expertise

*	Default values appear here:

$eolcom !

$IF NOT SETGLOBAL CCS		$SETGLOBAL CCS		0.95	! Cost of consulting services
$IF NOT SETGLOBAL GAMMA		$SETGLOBAL GAMMA	0.2	! Growth spillover
$IF NOT SETGLOBAL TMCS		$SETGLOBAL TMCS		0	! Tariff on imports of consulting services
$IF NOT SETGLOBAL CAPFLOW	$SETGLOBAL CAPFLOW	YES	! Capital flows 
$IF NOT SETGLOBAL SPILLOVER	$SETGLOBAL SPILLOVER    1
$IF NOT SETGLOBAL ESUBI		$SETGLOBAL ESUBI	2	! Substitution - high skilled labor
$IF NOT SETGLOBAL ESUBDM	$SETGLOBAL ESUBDM	2	! Substitution - domestic versus import
$IF NOT SETGLOBAL ETADX		$SETGLOBAL ETADX	2	! Substitution - domestic versus import
$IF NOT SETGLOBAL CRTS		$SETGLOBAL CRTS		NO	! Constant returns model

*=========================================================================================

*	Declare sets:

SET     COLS	SAM columns	/Y, X, IMPORT, INVEST, FD, MN, COLSUM/
        ROWS	SAM rows	/PD, PM, PX, PS, PFX, PL, RK, RB, MK, T, ROWSUM/;

*=========================================================================================

*	List input parameters:

SET	T		Time periods		/2000*2140/,
	TP(T)		Plot periods		/2000*2100/;

SCALAR
	PRIMAL		Flag for primal termination		/0/
	DUAL		Flag for dual termination criterion	/1/
	CAPFLO		Flag for capital flows
	BOPCON		Flag for balance of payments constraint	
        R		Baseline equilibrium interest rate	/0.05/
        G		Baseline equilibrium growth rate	/0.02/
	PHI		Adjustment cost parameter		/0/
	ESUBT   	Intertemporal elasticity		/0.5/
        ETADX		Elasticity of transformation (d vs x)   /%ETADX%/
        ESUBDM		Elasticity of substitution (d vs m)     /1/
        ESUBI		Elasticity of substitution (I)		/1/
        ALPHA		Intermediate value share                /0.66/
        THETAD		Domestic share of skilled labor	        /0.75/
        THETAF		Import share of demand		        /0.10/
	CRTS		Switch for CRTS submodel		

        THETA		Inverse intertemporal elasticity        
        LAMDA		Capital survival share                  /0.93/;

CRTS = 1$(%CRTS%);

SCALAR	KVS   		Capital value share in X		/  0.50 /;

*=========================================================================================

PARAMETER
	X0		User cost of intermediates(gross markup)
	L0		Labor demand, 
	E0		Exported production, 
	D0		Domestic production,
	MK0		Base year markup revenue,
	VK0		Value of capital inputs
	MX0		Imported inputs to intermediate, 
	M0		Aggregate base year imports (net of tariff),
	DX0		Domestic inputs to intermediate production, 
	CD0		Base year consumption (domestic),
	CM0		Base year consumption (imports);


SET	TL(T)		Decades	/2000,2010,2020,2030,2040,2050,2060,2070,2080,2090,2100/
	TFIRST(T)	Initial period
        TLAST(T)	Terminal period;

ALIAS (T,TT);
TFIRST(T) = YES$(ORD(T) EQ 1);
TLAST(T)  = YES$(ORD(T) EQ CARD(T));

PARAMETER
	PREF(T)		Reference price,
	QREF(T)		Reference quantity;

SCALAR
	F0		Base year value of firm equity, 
	I0		Base year new firm creation, 
	H0 		Entrepreneurial human capital stock
	IX0		Base year net investment in X-sector capital,
	JX0		Base year gross investment in X-sector capital,
	K0		Base year physical capital stock in X
	MKCHK		Cross check of markup revenue (should be all zero)
	C0		Base year final consumption (domestic plus imports),
	PK0		Capital price
	PKA0		Adjustment premium
        RK0		Net rate of return to capital (bae year user cost)
        UREF		Reference utility level
        TREV		Base year tariff revenue
        INVST		Base year investment
	GAMMA		Growth impact of imported consulting services /0/
	CCS		Cost of imported consulting services	/1.2/,

	BETA		Leisure ratio				/0/,
	GH		Growth rate of H 
	HL0		High-skilled labor associate with leisure demand
	LS0		Leisure demand value
	TMCS		Tariff rate on imports of consultin services /0/;

*=========================================================================================

*	Create initial SAM:

X0 = ALPHA;
L0 = 1 - ALPHA;
MK0 =    X0 * (1-ALPHA);
VK0 = KVS * (X0 - MK0);
DX0 = (1-KVS) * (X0 - MK0);

PK0 = (1+R)*(1+PHI*(G+1-LAMDA));
PKA0 = PHI * SQR(1-LAMDA+G)/2;
RK0 = PK0 * (R + 1-LAMDA)/(1+R) - PKA0;
K0 = VK0 / RK0;
DISPLAY R, LAMDA, RK0;

MKCHK = MK0 - (DX0+RK0*K0)*(1/ALPHA-1);
ABORT$ROUND(MKCHK,5) "Markup revenue does not match!",MKCHK;

F0  = MK0 * (1+R) / R;

*	New firm investments are consistent with steady-state:

I0  = G * F0 /(1+R);

*	We define H0 for the model with human capital:

GH = G * (1+BETA);
H0  = I0 * (1+R)/(R-GH);
HL0 = BETA * H0;
LS0 = HL0 * R/(1+R);

IX0  = K0 * (G + 1 - LAMDA);

*	Gross investment:

JX0 = IX0 * (1 + PHI * IX0/(2*K0));

INVST = I0 + JX0 - MK0 - VK0;
TREV = 0;

*       Calibrate final consumption of imports and domestic demand:

CM0 =    THETAF *(1-ALPHA-INVST+TREV+I0);
CD0 = (1-THETAF)*(1-ALPHA-INVST+TREV+I0);
C0      = CD0 + CM0;
DISPLAY C0, ALPHA, INVST, TREV;


*       Domestic output:

D0 = CD0 + DX0 + JX0;

*       Calibrate export levels to assure base year capital account balance:

E0 = CM0;
M0 = E0;


QREF(T) = (1 + G)**(ORD(T)-1);
PREF(T) = 1/(1+R)**(ORD(T)-1);

UREF = (C0+LS0) * SUM(T, QREF(T)*PREF(T));

$ONTEXT
$MODEL:romer

$SECTORS:
        U               ! Intertemporal utility
        Y(T)            ! Macro production (additively separable)
        X(T)            ! Aggregate intermediate
        I(T)$I0         ! Investment (creation of new firms)
        N(T)            ! Number of firms
        C(T)            ! Consumption 
        M(T)            ! Imports
	E(T)		! Exports
        MCS(T)          ! Imports of consulting services
        H(T)$H0         ! Level of human capital
	LS(T)$LS0	! Leisure
        K(T)            ! Capital stock in sector x
        IX(T)           ! Investment in sector x

$COMMODITIES:
        PU              ! Utility price 
        PD(T)           ! Domestic output price
        PC(T)           ! Consumption price
        PM(T)           ! Import price
	PE(T)		! Export price
	PLS(T)$LS0	! Leisure
        RK(T)           ! Rental price on capital
        PK(T)           ! Price of capital
	PKA(T)$PHI	! Adjustment cost
        PKT             ! Price of terminal capital
        PFX(T)$BOPCON   ! Real exchange rate
        PVFX$CAPFLO     ! Price of intertemporal foreign exchange
        PL(T)           ! Wage rate
        RHD(T)          ! Human capital rents - domestic
        RHM(T)          ! Human capital rents - multinational
        PH(T)$H0        ! Asset value of human capital
        PX(T)           ! Variety-adjusted price of X
        PF(T)$F0        ! Firm value
        PFT$F0          ! Price of terminal firms
        PHT$H0          ! Price of terminal ideas
        MK(T)           ! Markup value

$AUXILIARY:
        QADJUST(T)      ! Quantity adjustment
        PADJUST(T)      ! Price adjustment
        TK              ! Terminal capital
        TH$H0           ! Terminal stock of human capital
        TFIRM$F0        ! Terminal firm
	AT$CAPFLO	! Terminal net financial assets

$CONSUMERS:
        RA			! Representative agent
        ENTR(T)$(NOT CRTS)	! Entrepreneur

$PROD:U  s:esubt
        O:PU            Q:UREF          
        I:PC(T)         Q:((C0+LS0)*QREF(T)) P:PREF(T)
        
$PROD:C(T) s:1  dm:esubdm
        O:PC(T)         Q:(C0+LS0)
	I:PLS(T)	Q:LS0  P:PREF(T)
        I:PD(T)         Q:CD0  P:PREF(T)	dm:
        I:PM(T)         Q:M0   P:PREF(T)	dm:

*	Imports of final consumption:

$PROD:M(T)
        O:PM(T)			Q:M0  
        I:PFX(T)$BOPCON		Q:M0            
        I:PVFX$CAPFLO		Q:(M0*PREF(T))  

$PROD:E(T)
        O:PFX(T)$BOPCON		Q:E0
        O:PVFX$CAPFLO		Q:(E0*PREF(T))
	I:PE(T)			Q:E0

*	Assembly:

$PROD:Y(T)   s:1  t:etadx
        O:PD(T)         Q:D0            P:PREF(T)       
	O:PE(T)		Q:E0		P:PREF(T)
        I:PX(T)		Q:X0            P:PREF(T)
	I:PL(T)		Q:L0		P:PREF(T)

*	Production:

$PROD:X(T)$(NOT CRTS)  s:0
        O:PX(T)         Q:X0		A:RA    N:PADJUST(T)  M:-1
	I:PD(T)		Q:DX0		P:PREF(T) A:ENTR(T) T:(1/ALPHA-1)
        I:RK(T)         Q:(RK0*K0)	P:PREF(T) A:ENTR(T) T:(1/ALPHA-1)

$PROD:X(T)$CRTS		s:0
        O:PX(T)         Q:X0	
	I:PD(T)		Q:DX0		P:PREF(T) 
        I:RK(T)         Q:(RK0*K0)	P:PREF(T) 
        I:MK(T)         Q:MK0		P:PREF(T) 

$PROD:K(T)
        O:PK(T+1)       Q:(LAMDA*K0)
        O:PKT$TLAST(T)  Q:(LAMDA*K0)
        O:RK(T)         Q:(RK0*K0)  
	O:PKA(T)	Q:(PKA0*K0)
        I:PK(T)         Q:K0

$PROD:IX(T)  s:0  adj:1
        O:PK(T+1)       Q:IX0
        O:PKT$TLAST(T)  Q:IX0
        I:PD(T)         Q:IX0
	I:PKA(T)	Q:(PKA0*K0) adj:
	I:PD(T)		Q:(PKA0*K0) adj:


$PROD:N(T)
        O:PF(T+1)       Q:F0
        O:PFT$TLAST(T)  Q:F0
        O:MK(T)         Q:MK0
        I:PF(T)         Q:F0         

$PROD:I(T)$I0  s:esubi
        O:PF(T+1)       Q:(I0*(1+R))
        O:PFT$TLAST(T)  Q:(I0*(1+R))
        I:RHM(T)        Q:((1-THETAD)*I0)
	I:RHD(T)	Q:(THETAD*I0)

$PROD:H(T)$H0 
        O:PH(T+1)       Q:(H0*(1+GH))
        O:PHT$TLAST(T)  Q:(H0*(1+GH))
        O:RHM(T)        Q:((1-THETAD)*I0)
        O:RHD(T)        Q:(THETAD*I0)
        I:PH(T)         Q:H0 

$PROD:LS(t)$LS0
	O:PLS(T)	Q:LS0
	O:PH(T+1)	Q:HL0
        O:PHT$TLAST(T)  Q:HL0
	I:PH(T)		Q:HL0

$PROD:MCS(T)
        O:PH(T+1)	Q:(H0*GAMMA) A:RA  T:(%SPILLOVER%)
        O:RHM(T)	Q:I0
        I:PM(T)         Q:(I0*CCS)   A:RA  T:TMCS

$DEMAND:RA
        E:PH(TFIRST)	Q:(H0+HL0)
	E:RHD(T)$(NOT H0)  Q:(QREF(T)*I0*THETAD)
	E:RHM(T)$(NOT H0)  Q:(QREF(T)*I0*(1-THETAD))
        E:PL(T)		Q:(QREF(T)*L0)
        E:PF(TFIRST)    Q:F0
        E:PK(TFIRST)    Q:K0
        E:PX(T)         Q:X0		R:QADJUST(T)
        E:PFT	        Q:(-F0)		R:TFIRM
        E:PKT           Q:(-K0)		R:TK
        E:PHT           Q:(-(H0+HL0) )	R:TH
	E:PVFX$CAPFLO	Q:(-1)		R:AT
	D:PU            Q:UREF

$CONSTRAINT:AT$CAPFLO
	AT =E= (1+G)/(R-G) * SUM(TLAST(T), 
		C(T)*PC(T)*C0 
		- PL(T)*QREF(T)*L0 
		- (RHD(T)*QREF(T)*I0)$(NOT H0)
		- PHT*TH*(H0+HL0)  
		- PFT*TFIRM*F0 
		- PKT*TK*K0);

$DEMAND:ENTR(T)$(NOT CRTS)
        D:MK(T)       Q:MK0

$CONSTRAINT:TFIRM$F0
         SUM(T$TLAST(T+1), C(T)*I(T+1) - I(T)*C(T+1))$PRIMAL
       + SUM(T$TLAST(T+1), PFT * PC(T) - PF(T+1) * PC(T+1))$DUAL =E= 0;

$CONSTRAINT:TK
         SUM(T$TLAST(T+1), C(T)*IX(T+1) - IX(T)*C(T+1))$PRIMAL
      +  SUM(T$TLAST(T+1), PKT * PC(T)  - PK(T+1) * PC(T+1))$DUAL  =E= 0;

$CONSTRAINT:TH$H0 
         SUM(T$TLAST(T+1),  C(T)*(H(T+1)+MCS(T+1)*H0 /I0) 
                - (H(T)+MCS(T)*H0 /I0)*C(T+1))$PRIMAL
       +  SUM(T$TLAST(T+1), PHT * PC(T) - PH(T+1) * PC(T+1))$DUAL  =E= 0;

$CONSTRAINT:QADJUST(T)
        QADJUST(T) =E= X(T) * PADJUST(T);

$CONSTRAINT:PADJUST(T)
        1 + PADJUST(T) =E= (N(T)/QREF(T))**(1/ALPHA - 1);

$OFFTEXT
$SYSINCLUDE mpsgeset romer

*=====================================================================================

*       Assign initial level values:

PU.L      = 1;
PD.L(T)   = PREF(T);
RHM.L(T) = PREF(T);
RHD.L(T) = PREF(T);
PH.L(T) = PREF(T);
PLS.L(T) = PREF(T);
PC.L(T)   = PREF(T);
PFX.L(T)  = PREF(T);
PM.L(T)   = PREF(T);
PE.L(T)   = PREF(T);
PL.L(T) = PREF(T);
PX.L(T) = PREF(T);

RK.L(T) = PREF(T);
PK.L(T) = PREF(T) * PK0;
PKA.L(T) = PREF(T);
PF.L(T) = PREF(T);
MK.L(T) = PREF(T);

LS.L(T) = QREF(T);
E.L(T)    = QREF(T);
M.L(T)    = QREF(T);
X.L(T)  = QREF(T);
Y.L(T)  = QREF(T);
C.L(T)    = QREF(T);
I.L(T)  = QREF(T);
IX.L(T) = QREF(T);
K.L(T) = QREF(T);
H.L(T) = QREF(T);
MCS.L(T) = 0;

N.FX(T)$CRTS      = QREF(T);
N.L(T)$(NOT CRTS) = QREF(T);

PADJUST.L(T) = 0;
QADJUST.L(T) = 0;

PKT.L   = PK0 * (1/(1+R))**CARD(T);

LOOP(TLAST,
        PFT.L   = PREF(TLAST)/(1+R);
        TFIRM.L = QREF(TLAST) * (1 + G);
        PHT.L   = PREF(TLAST) / (1+r);
        TK.L    = QREF(TLAST) * (1 + G);
        TH.L    = QREF(TLAST) * (1 + G);
);

PADJUST.LO(T) = -0.8;

QADJUST.UP(T) = +INF;
QADJUST.LO(T) = -INF;
QADJUST.FX(TFIRST) = 0;
PADJUST.FX(TFIRST) = 0;

*=====================================================================================

*       Check benchmark consistency:

BOPCON = YES;
CAPFLO = NO;
AT.FX = 0;

TH.FX$(NOT H0) = 0;
PU.FX = 1;
GAMMA = %gamma%;

*	When spillover=1, TMCS=0

TMCS = (H0/I0) * GAMMA * (1-%SPILLOVER%) / ((1+R)*CCS);

ROMER.ITERLIM = 0;
$INCLUDE ROMER.GEN
SOLVE ROMER USING MCP;
ROMER.ITERLIM = 8000;

*=====================================================================================

*	Apply the shock:

CCS = %ccs%;
TMCS = %TMCS%;
CAPFLO = YES$%CAPFLOW%;
BOPCON = YES$(NOT CAPFLO);

*	Compute the equilibrium first with ESUBI and ESUBDM=1:

ESUBI=1.01;
ESUBDM=1.01;
ROMER.ITERLIM = 8000;
$INCLUDE ROMER.GEN
SOLVE ROMER USING MCP;

if ((%esubi% ne 1) or (%esubdm% ne 1),
	ESUBI = %ESUBI%;
	ESUBDM=%ESUBDM%;
$INCLUDE ROMER.GEN
	SOLVE ROMER USING MCP;
);

abort$(romer.solvestat ne 1) "Solve status is not 1!";
abort$(romer.modelstat ne 1) "Model status is not 1!";

parameter       evterm  EV through model horizon,
                evpost  Post-terminal EV,
                theta_m Value share of infra-model consumption,
                rho     Primal elasticity parameter,
                evtotal EV through infinite horizon,
		pvtmcs	Present value of tax on MCS (% of total consumption)
                g2010   Growth through 2010,
                g2050   Growth through 2050,
                report  Summary report of model results;



*       Generate a report of results for this scenario:

evterm = 100 * (u.l - 1);
evpost = sum(tlast, c.l(tlast)/qref(tlast));
theta_m = 1 - ((1+g)/(1+r))**card(t);
rho = 1 - 1/esubt;
evtotal = 100 * ((theta_m*u.l**rho+(1-theta_m)*evpost**rho)**(1/rho)-1);
pvtmcs = 100 * sum(t, tmcs * i0 * ccs * mcs.l(t) * pm.l(t)) / 
	 (pu.l * uref);
evpost = 100 * (sum(tlast, c.l(tlast)/qref(tlast)) - 1);;
g2010 = 100 * (c.l("2010")**(1/10.0) - 1);
g2050 = 100 * (c.l("2050")**(1/50.0) - 1);

set	reprow	Rows included in tabular reports /
	evterm,evpost,evtotal,pvtmcs,g2010,g2050,modelstat,solvestat/;

report("evterm") = evterm;
report("evpost") = evpost;
report("evtotal") = evtotal;
report("pvtmcs") = pvtmcs;
report("g2010") = g2010;
report("g2050") = g2050;
report("modelstat") = romer.modelstat;
report("solvestat") = romer.solvestat;

parameter       wage            Domestic wages (% change)
                consum          Consumption (% change)
                invest          Physical investment (% change)
                firms           Firms (% change)
                invest          Investment
		intrate		Interest rate (%)
                erate           Exchange rate (% change)
                rent            Rents to R&D inputs (% change),
                hreturn         Return to human capital (% change),
                consult         Foreign consultant share of R&D market (%),
		output		Output per firm in the X sector (% change)
		growth		Consumption growth rate (% change)
		curracct	Current account deficit (% imports)
                firmentry       Entry of firms	
		pcapital	Physical capital
		hcapital	Human capital
		kcapital	Knowledge capital
		rhdm		Ratio of RHD to RHM
		pvpfx		Present value of foreign exchange;



rhdm(t) = rhd.l(t)/rhm.l(t);
intrate(t) = r;
intrate(t)$pfx.l(t+1) = 100 * (pfx.l(t)/pfx.l(t+1) - 1);
growth(t) = na;
growth(t)$c.l(t+1) = 100 * (c.l(t+1)/c.l(t)-1-g);
output(t) = 100 * (X.L(t)/N.L(t) - 1);
wage(t) = 100 * (pl.l(t)*l0 / (pc.l(t)*l0) - 1);
consum(t) = 100 * (c.l(t)/qref(t) - 1);
invest(t) = 100 * (( ix.l(t)*ix0)/(qref(t)*ix0) - 1);
firms(t) = 100 * (n.l(t)/qref(t) - 1);
invest(t)  = 100 * (ix.l(t)/qref(t) - 1);
curracct(t) = ( 100 * (1-E.L(T)/M.L(T)) )$CAPFLO;
pvpfx(t) = (pfx.l(t)/pfx.l("2000"))$bopcon + (pvfx.l*pref(t))$capflo;

if (capflo,erate(t) = 0; else erate(t) = 100 * (pm.l(t)/pc.l(t)-1););

rent(t) = 100 * ((thetad*rhd.l(t)+(1-thetad)*rhm.l(t))/pc.l(t)-1);
hreturn(t) = 100 * ((thetad*rhd.l(t)+(1-thetad)*rhm.l(t))/pc.l(t) * h.l(t)/qref(t)-1);
consult(t) = 100 * (mcs.l(t) / (mcs.l(t) + h.l(t)));
firmentry(t) = 100 * (i.l(t)/qref(t)-1);

pcapital(t) = 100 * (k.l(t)/qref(t)-1);
kcapital(t) = 100 * (n.l(t)/qref(t)-1);
hcapital(t) = 100 * (h.l(t)/qref(t)-1);


display report, wage, consum, invest, firms, invest, erate, 
        rent, hreturn, consult, firmentry, output;

*       Define sets for displaying figures -- all figures must have the same 
*       domain and labels:

parameter       welfare         Consumption and welfare impacts (%);

welfare(t,"c") = consum(t);
welfare(t,"evterm") = report("evterm");
welfare(t,"evpost") = report("evpost");
welfare(t,"evtotal") = report("evtotal");

$libinclude writesol report wage consum invest firms erate rent hreturn consult firmentry pcapital kcapital hcapital curracct pvpfx rhdm intrate
$exit


$setglobal domain tp
$setglobal labels tl
$libinclude plot rent
$libinclude plot hreturn
$libinclude plot curracct
$libinclude plot consult
$libinclude plot intrate
$libinclude plot growth
$libinclude plot output
$libinclude plot welfare
$libinclude plot firmentry
$libinclude plot wage
$libinclude plot invest
$libinclude plot firms
$libinclude plot erate

$exit

