@echo off

rem	GAMS Scenario Interface SubProgram -- table.bat
rem	Thomas F. Rutherford
rem	February, 2001

rem	This batch command file can be used to define and generate
rem	tabular reports for GAMS models which have been solved using
rem	the GAMS-SM package.

rem	Command syntax for generating a table:
rem
rem		table ident item [nd=xx] sc1 [sc2 ... sc8]
rem
rem	where	ident		is the table identifier
rem		item		is the GAMS-SM item to include in the table
rem		nd=xx		indicates the table be displayed with xx decimals
rem		sc1..sc8	are identifiers for solved scenarios (up to 8)

rem	Define whether Excel output is generated by default with the
rem	following statement  (xls=yes or xls=no):

set xls=no

if "%1"=="" goto syntax
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

set ident=%1
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

rem	Be sure that the table has been written to the solution file:

if not exist output\items\%1 goto noitem

echo	$setglobal ident %ident%	>output\%ident%.gms
echo	$setglobal item %1		>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if not %1==/nd goto cases
echo	$setglobal decimals %2		>>output\%ident%.gms

shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

:cases
if "%1"==""	goto nocolumns

if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc1 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc2 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc3 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc4 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc5 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc6 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc7 %1	>>output\%ident%.gms
shift
if "%1"=="/xls" set xls=yes
if "%1"=="/xls" shift

if "%1"=="" goto generate
if not exist scenarios\%1 goto noscenario
if not exist output\%1.sol goto nosolution
echo	$setglobal sc8 %1	>>output\%ident%.gms

:generate
if %xls%==yes	echo	$setglobal xls yes	>>output\%ident%.gms
echo	$libinclude table.gms			>>output\%ident%.gms
if     exist inclib\nul call gams output\%ident%.gms o=output\%ident%.lst relpath=1 ldir=.\inclib
if not exist inclib\nul call gams output\%ident%.gms o=output\%ident%.lst 

goto end

:nocolumns
echo.
echo	Error:  no scenarios specified for table.
echo.
pause
goto syntax
goto end

:noscenario
echo.
echo	Error:	Table column %1 is not a scenario.
echo.
echo	Must be one of the following:
echo.
dir /b scenarios
echo.
goto end

:nosolution
echo.
echo	Error:	table scenario %1 has not been solved.
echo.
goto end

:noitem
echo.
echo	Error:	Table %1 is not recognized.
echo.
echo	Must be one of the following:
echo.
dir /b output\items
echo.
goto end

:syntax
echo.
echo	Syntax:		table [/xls][/nd=xx] ident item sc1 [sc2 ... sc8]
echo.
echo	where		ident		is the table identifier
echo			item		is the item to include in the table
echo			/xls		Indicates table to be saved in output\ident.xls
echo			/nd=xx		Indicates table displayed with xx decimals
echo			sc1..sc8	are scenario identifiers (up to 8)
echo.
echo.
:end
